/* This file contains the definition and methods for the
 * RTPPktLogger class.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inet.h"
#include "rtp.h"
#include "rep-decoder.h"
#include "bsd-endian.h"
#include "tclcl.h"
#include "renderer.h"
#include "pktbuf.h"
#include "vidreps.h"
#include "logger.h"

/*
 * The RTPPktLogger class corresponds to the OTcl class
 * Module/RTPPktLogger. This PacketHandler derived class
 * is used to log RTP packets with the global LogService
 * object described in logger.cc. Any packet that arrives
 * on the recv method is logged with the LogService and
 * simply forward to the next target.
 */

class RTPPktLogger : public PacketHandler {
public:
  RTPPktLogger() : PacketHandler(sizeof(rtphdr)) {};
  
  virtual void recv(pktbuf *pb);
};

/*
 * Recv entry method for RTP packets.
 */

void 
RTPPktLogger::recv(pktbuf *pb) 
{
  logger.log_rtp_pkt(pb);
  target_->recv(pb);
}	
		
/*
 * Binding between C++ class RTPPktLogger and
 * OTcl class "Module/RTPPktLogger"
 */

static class RTPPktLoggerClass : public TclClass {
public:
  RTPPktLoggerClass() : TclClass("Module/RTPPktLogger") {}
  TclObject *create(int argc, const char*const* argv) {
    return (new RTPPktLogger());
  }
} swd_rtp_pkt_logger_;

