#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/avswitch-server/avswitch-srvr.tcl,v 1.5 1998/07/22 04:35:56 hodes Exp $
#


import SerialChannel/AVSwitch-Knox-RS8x8 UDPServer

# AVSwitch RPC-style interface
#
Class UDPServer/AVSwitch -superclass UDPServer

UDPServer/AVSwitch public init {mtu addrspec parent} {
    $self instvar parent_ pending_announce_
    set parent_ $parent
    set pending_announce_ -1

    eval [list $self] next $addrspec $mtu
}


UDPServer/AVSwitch private recv {addr port data size} {
    puts "Msg: $addr\[$size\]: $data"
    
    $self instvar parent_ pending_announce_

    set theMethod [lindex $data 0]
    switch $theMethod {
	refresh_settings {[$parent_ set avswitch_] refresh_settings}
	test_LEDs {[$parent_ set avswitch_] test_LEDs}
	route_both -
	route_audio -
	route_video {$parent_ set_routes "$data"}
    }
    if {$pending_announce_ != -1} {
	after cancel $pending_announce_
    }
    # have to wait for the update from the switch to be received & processed
    set pending_announce_ [after 4000 $self announce_reply]
}

UDPServer/AVSwitch private announce_reply {} {
    $self instvar parent_ pending_announce_

    set av [$parent_ set avswitch_]
    set v [$av array get videoSettings_]
    set a [$av array get audioSettings_]
    $self announce "v: $v ; a: $a"
    set pending_announce_ -1
}


#
# a UI-less server that accepts RPC-like commands and
# returns results for controlling an avswitch
#
Class AVSwitchServer

AVSwitchServer public init {addrspec} {
    $self instvar avswitch_ al_

    set avswitch_ [new SerialChannel/AVSwitch-Knox-RS8x8 1024]
    set al_ [new UDPServer/AVSwitch 1400 $addrspec $self]
}

AVSwitchServer public destroy {} {
    $self instvar al_ avswitch_

    delete $al_
    delete $avswitch_

    eval [list $self] next
}

AVSwitchServer public set_routes {args} {
    $self instvar avswitch_
    
    set args [lindex $args 0]   ; # this is necessary 'cause param is "args"
    #puts "args: $args"
    
    if {[llength $args] != 3} {
	puts "bad \# of args to set-routes"
	return
    }
    set cmd [lindex $args 0]
    set out [lindex $args 1]
    set in [lindex $args 2]

    if {$out > 8 || $out < 1 || $in > 8 || $in < 1} {
	puts "bad values to set-routes"
	return
    }
    
    eval $avswitch_ $args
}
