#
# Copyright (c) 1997-1998 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# 	This product includes software developed by the MASH Research
# 	Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/pathfinder/mash_server.tcl,v 1.2 1998/10/05 20:48:12 yatin Exp $
#

import HTTP_Server


#
# The MASH_Server class is a basic HTTP server that is geared towards
# simplifying the Webifying of MASH tools. The server can contain any
# number of HTTP_Agent objects each of which handle certain client
# requests. These agents are added through the add_agent method. Refer
# to the abstract HTTP_Agent class for specifics on implementing agents.
#
Class HTTP_Server/MASH_Server -superclass HTTP_Server


HTTP_Server/MASH_Server public init { } {

    $self next
    $self instvar html_dir_

    # Initialize the html directory.
    set html_dir_ ~/mash-1/tcl/applications/mash_server/html/
    #set o [$self options]
    #$o load_preferences "mserver"
    set html_dir_ [$self get_option html_dir]
}


#
# The add_agent method takes an HTTP_Agent object and adds it to the
# current list of agents managed by the MASH_Server.
#
HTTP_Server/MASH_Server public add_agent { agent } {
    $self instvar agents_
    lappend agents_ $agent
}


#
# When the MASH_Server receives a client HTTP request, the handle_request
# method is called. In turn, the server looks to each of its agents and
# has each process the request until one of the agents handles the
# request. The data returned by the agent is then returned to the client
# in the form of an HTTP response.
#
HTTP_Server/MASH_Server public handle_request { socket headers data } {

    $self instvar agents_ html_dir_

    set url [string trimleft [$self extract_url $headers] /]
    set key [$self extract_key $url]
    set data ""
    set isRawData 0
    set page { }
    set type "text/html"

    # Find out the source of this request.
    set source [$self find_source $socket]

    foreach a $agents_ {
	# A page is returned if the url was recognized by the
	# agent.  In this case, break out of the for loop.
	set page [$a handle_request $url $key $source]
	set data [$self get_data $page]
	if { $data != "" } {
	    break
	}
    }
    
    puts $url
    if { [string length $url] == 0 } {
	set url mash_server.html
    }

    if { $data == "" } {
	if { [string match "*.html" $url] || [string match "*.txt" $url] } {
	    mtrace trcNet "-> Nondynamic page requested."
	    append html_file $html_dir_ $url
	    set data [read_file $html_file]
	    set page [list $data 200 $type]
	    
	} else {
	    mtrace trcNet "-> Data requested."
	    append data_file $html_dir_ $url
	    set data [read_file $data_file]
	    set isRawData 1
	}
    }

    # This whole section is a hack since otcl cannot handle binary data.
    if { $isRawData == 1 } {
	set status "HTTP/1.0 200 OK"
	set data_len [string length $data]
	set header_list [list "content-type" "image/gif" \
		"content-length" $data_len]
	set headers [$self construct_headers $status $header_list]
	$socket send $headers

	# Configure the socket to binary, non-blocking mode. ???
	$socket set_binary
	set chan [$socket channel]
	fconfigure $chan -blocking 0
	fconfigure $chan -translation cr

	puts -nonewline $chan $data
	flush $chan

	$socket close
	$socket shutdown

    } else {
	set status [$self get_status $page]
	set type [$self get_type $page]
	$self send_reply $socket $data $type $status
    }
}


#
# The get_data method extracts the data portion of the page returned
# by the agent.
#
HTTP_Server/MASH_Server private get_data { page } {
    return [lindex $page 0]
}


#
# The get_status method extracts the status portion of the page 
# returned by the agent.
#
HTTP_Server/MASH_Server private get_status { page } {
    return [lindex $page 1]
}


#
# The get_type method extracts the status portion of the page 
# returned by the agent.
#
HTTP_Server/MASH_Server private get_type { page } {
    return [lindex $page 2]
}


#
# The extract_key method extracts the key from the magic urls.  The 
# standard form of a magic URL is [description]^[unique key].
#
HTTP_Server/MASH_Server private extract_key { url } {
    set offset [string last ^ $url]
    set key [string range $url [expr $offset + 1] \
	    [string length $url] ]
    return [string trimleft [string tolower $key] -:]
}


#
# The find_source method uses the socket to determine the source
# of a HTTP request.
#
HTTP_Server/MASH_Server private find_source { socket } {

    # Find the hostname of the client.
    set chan [$socket channel]
    set host [lindex [fconfigure $chan -peername] 1]
    mtrace trcNet "-> Client hostname: $host"

    return $host
}

# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


proc write_to_file { filename string } {
    set fileid [open $filename "WRONLY CREAT TRUNC"]
    puts $fileid $string
    close $fileid
}


proc file_dump { filename } {
    set fileid [open $filename "r"]
    set data [read $fileid]
    close $fileid
    return $data
}


proc prepend_to_file { filename string } {

    # Read the original data in the file
    set orig [read_file $filename]

    # Truncate all the data in the file
    set fileid [open $filename "WRONLY CREAT TRUNC"]
    close $fileid

    # Prepend string to the file
    set fileid [open $filename "WRONLY CREAT APPEND"]
    puts $fileid $string
    puts $fileid $orig
    close $fileid
}


proc read_file { filename } {
    set exists [file exists $filename]
    set data ""
    if { $exists == 1 } {
	set fileid [open $filename "r"]
	set data [read $fileid]
	close $fileid
    }
    return $data
}


proc get_key { program } {
    return [string tolower [string trimleft [$program unique_key] -:]]
}


proc edit_html { str } {

    # Replace "<" and ">" so that they are not specially treated if
    # there is an email address within them.
#    regsub -all -- (<)(\[^>\]*)(@)(\[^>\]*)(>) $str {\&lt\2\3\4\&gt} str
    regsub -all -- < $str {\&lt} str
    regsub -all -- > $str {\&gt} str

    return $str
}

