#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/room-manager/room-manager.tcl,v 1.4 1998/08/26 04:42:28 yatin Exp $
#

# ------
#
#  WARNING: This file is a bunch of stubs!
#

import AnnounceListenManager


#
# Maintains information about resources, objects, agents in a 
# manually-configured domain.  Announces itself to a well-known
# address, then accepts attribute queries, resolves them to
# a single best match (heuritically if necessary), and replies
# with the all the data (atrributes) of the best match and some
# indication of how many other matches there where and possibly
# even their names.     XXX XXX -- to be determined 
# <p>
# the well-known port for roomManager broadcast/multicasts
# is 1202 (tenatively).
#
Class RoomManager/Server

RoomManager/Server public init {parent} {
	$self instvar al_ parent_

	set parent_ $parent
	set al_ [new ALM/RoomManager/Server 255.255.255.255/1202 $self]

	# now want to have an agent that keeps a cache of local services
	# and pointers to other room managers that it knows about
	# .. for now, ugly hard-coded in recv_announcement XXX
}


#
# communications management for the RoomManager server-side
#
Class ALM/RoomManager/Server -superclass AnnounceListenManager

ALM/RoomManager/Server public init {addr parent {mtu 1500}} {
	eval [list $self] next $addr $mtu
	$self instvar parent_
	set parent_ $parent
}

#
# builds periodic announcement of room_manager name and rendezvous address
#
ALM/RoomManager/Server private send_announcement {} {
	$self instvar port_
	set la [localaddr]
	announce "room_manager: $la/$port_" ;# XXX
}

ALM/RoomManager/Server private recv_announcement {addr port data size} {
	puts "Msg: $addr/$port\[$size\]: $data"
}

#
# communications management for the RoomManager client-side
#
Class ALM/RoomManager/Client -superclass AnnounceListenManager


ALM/RoomManager/Client private recv_announcement {addr port data size} {
	puts "Msg: $addr/$port\[$size\]: $data"
}


#
# client-side agent for finding and communicating with the "local"
# roomManager.  Basically acts like a directory.
#
Class RoomManager/Client


RoomManager/Client public init {{addrspec 255.255.255.255/1202} {mtu 1500}} {
	$self instvar al_

	set al_ [new ALM/RoomManager/Client $addrspec $mtu]
	$al_ set parent_ $self
	
}

#
# ask for info on obj specified by list of attributes ("args")
# <p>
# attributes are in one of twp forms, much like URLs/unix FS names:
#<p>
# attribType:/attr/subattr
#<p>
# attribType:subattr/subsubattr
#<p>
# where a attrib can be a "*" for wildcard
#
RoomManager/Client public get_object_info {attribStr} {

	set data [$self get_object_info_from_local_cache $attribStr]
	if {$data == ""} {
		set data [$self get_object_info_from_server $attribStr]
	}

	if {$data == ""} {
		puts "No data exists matching attributes:\n$attribStr\n"
	}
	
	return $data
}

#
# currently just a stub -w- some hard-coded info from 326
#
RoomManager/Client public get_object_info_from_local_cache {attribStr} {

	# XXX
	# This should do all the wildcard processing etc.

	switch $attribStr {
		"name:red" {
			return "type:SerialChannel/Camera-Sony-EVID30\ndevice:/dev/cuac00"
		}
		"name:yellow" {
			return "type:SerialChannel/Camera-Canon-VCC1\ndevice:/dev/cuac01"
		}
		"name:blue" {
			return "type:SerialChannel/Camera-Sony-EVID30\ndevice:/dev/cuac02"
		}
		"name:green" {
			return "type:SerialChannel/Camera-Sony-EVID30\ndevice:/dev/cuac03"
		}
	}
}

RoomManager/Client public get_object_info_from_server {attribStr} {
}

RoomManager/Client public get_attrib_data {obj attrib} {

	set data [$self get_object_info $obj]
	if {$data == ""} {return ""}
	set attribList [split $data "\n"]
	foreach i $attribList {
		set idx [string first $attrib $i]
		if {$idx != -1} {
			return "[lindex [split $i ":"] 1]"
		}
	}
}
