
# This file defines the Parameter class and many subclasses.
# Each subclass of Parameter must provide implementations for
# the following methods:
#
# get
# set 
# type
# domain

Class Parameter;

Parameter instproc init args {
    eval $self next $args;
}

Class RealParameter -superclass Parameter

RealParameter instproc init args {
    $self instvar from_ to_ current_;
    set from_ 0.0;
    set to_ 1.0;
    set current_ 0.0;
    eval $self next $args;
}

RealParameter instproc from args {
    $self instvar from_;

    if {[llength $args] == 1} {
	set from_ $args;
    }
    return $from_;
}

RealParameter instproc to args {
    $self instvar to_;

    if {[llength $args] == 1} {
	set to_ $args;
    }
    return $to_;
}

RealParameter instproc get {} {
    $self instvar current_;

    return $current_;
}

RealParameter instproc set value {
    $self instvar current_ from_ to_;

    if {$value < $from_} {
	set current_ $from_;
    } elseif {$value > $to_} {
	set current_ $to_;
    } else {
	set current_ $value;
    }
    return $current_;
}

RealParameter instproc type {} {
    return "real";
}

RealParameter instproc domain {} {
    $self instvar from_ to_;

    return "($from_,$to_)"
}

Class IntParameter -superclass Parameter

IntParameter instproc init args {
    $self instvar from_ to_ current_;
    
    set from_ 0;
    set to_ 1;
    set current_ 0;
    eval $self next $args;
}

IntParameter instproc from args {
    $self instvar from_;

    if {[llength $args] == 1} {
	set from_ [expr int($args)];
    }
    return $from_;
}

IntParameter instproc to args {
    $self instvar to_;

    if {[llength $args] == 1} {
	set to_ [expr int($args)];
    }
    return $to_;
}

IntParameter instproc get {} {
    $self instvar current_;

    return $current_;
}

IntParameter instproc set value {
    $self instvar current_ from_ to_;

    if {$value < $from_} {
	set current_ $from_;
    } elseif {$value > $to_} {
	set current_ $to_;
    } else {
	set current_ [expr int($value)];
    }
    return $current_;
}

IntParameter instproc type {} {
    return "int";
}

IntParameter instproc domain {} {
    $self instvar from_ to_;

    return "($from_,$to_)"
}

