# This file contains the TemporalInterleavor class
# The TmeporalInterleavor object responds to graph interface communication
# recieved through its recv_graph_comm method and sends graph
# interface communication throught the send_graph_comm method.

# Object fields
# 
# output_id_list_ : list of output ids. The same ids are expected to be
#                   used by individual execution subgraphs
# output_info_ : array of info on output_ids. For each output id $o,
#       ($o,spec) : multicast spec for where interleaved output data should
#                   go.
#       ($o,inter_obj) : interleaver object for incoming streams for this
#                        output id
#       ($o,vagent) : video agent for incoming streams for this output id
#       ($o,in_spec) : spec for incoming streams for this output id
#       ($o,out_vagent) : video agent for outgoing stream for this output id
# vagent_array_ : array associated by "addr,port" indexes that holds the
#                 name of video agents used by this object.
# subgraph_list_ : list of subgraph identifiers. These identifiers are local
#                  only
# subgraph_info_ : array of info on subgraphs. For each subgraph $s,
#      ($s,comm_obj) : graph comm object for talking to this subgraph
#      ($s,spec)     : spec of subgraph
# id_ : id of this object to be used in graph communication
# comm_obj_ : object for graph comm (from above, NOT to subgraphs)
# callback_array_ : holds comm callbacks with index <comm_obj>,<mid>

import PsvpVideoAgent
import AnnounceListenManager/GraphComm

Class TemporalInterleaver

TemporalInterleaver instproc init {id control_spec} {
    $self next;

    $self instvar output_id_list_;
    $self instvar output_info_;
    $self instvar vagent_array_;
    $self instvar subgraph_list_;
    $self instvar subgraph_info_;
    $self instvar id_;
    $self instvar comm_obj_;
    $self instvar callback_array_;
    $self instvar sid_;

    set output_id_list_ "";

    set id_ $id;
    set comm_obj_ [new AnnounceListenManager/GraphComm [$self get_option mtu] $control_spec];
    $comm_obj_ register $id "$self recv_graph_comm";

    set sid_ -1;
    set subgraph_list_ "";
}

TemporalInterleaver instproc send_to_subgraph {sid rid mesg {resend_period 0} {deliver_once_flag 0}} {
    $self instvar id_;
    $self instvar subgraph_info_;

    set comm_obj $subgraph_info_($sid,comm_obj);
    $comm_obj do_announce $id_ $rid $mesg $resend_period $deliver_once_flag;
}

TemporalInterleaver instproc recv_subgraph_comm {sid rid mesg} {
    $self instvar callback_array_;

    if {[info exists callback_array_($sid,$rid)]} {
	set cb $callback_array_($sid,$rid);
	unset callback_array_($sid,$rid);
	eval $cb [list $mesg];
    }
}

TemporalInterleaver instproc setup_outputs {sid req_id reply} {
    $self instvar subgraph_list_;
    $self instvar subgraph_info_;
    $self instvar vagent_array_;

    set comm_obj $subgraph_info_($sid,comm_obj);
    $comm_obj stop_resend $req_id;

    set out_list [lindex $reply 1];

    $self instvar output_id_list_;
    $self instvar output_info_;

    set output_id_list_ $out_list;

    $self instvar last_mesg_array;

    foreach o $output_id_list_ {
	set output_info_($o,spec) "";

	set spec [$self GenerateSpec];
	set spec_split [split $spec "/"];
	set addr [lindex $spec_split 0];
	set port [lindex $spec_split 1];

	set output_info_($o,inter_obj) [new Module/Combine/Temporal];
	set vagent_array_($addr,$port) [new TemporalInterleaverVideoAgent $addr/$port $output_info_($o,inter_obj)];
	set output_info_($o,vagent) $vagent_array_($addr,$port);
	set output_info_($o,in_spec) $addr/$port
	set output_info_($o,out_vagent) "";

	foreach s $subgraph_list_ {
	    # Create transmitting video agents in each subgraph and
	    # set spec accordingly

	    $self send_to_subgraph $s -1 [list set_output_spec $o $addr/$port] 1000 1;
	}
    }
}

TemporalInterleaver instproc send_graph_comm {rid mesg} {
    $self instvar comm_obj_;
    $self instvar id_;
    
    $comm_obj_ do_announce $id_ $rid $mesg;
}

TemporalInterleaver instproc recv_graph_comm {rid mesg} {
    eval $self instvar [$self info vars];

    set cmd [lindex $mesg 0];

    # Don't do anything if we don't have any subgraphs unless cmd
    # is add_subgraph
    # XXX This might be a problem since underlying comm object
    #     thinks we've handled this message. Maybe we need
    #     a way to tell comm object not to assume we've handled
    #     the message, or queue messages that we get when we weren't
    #     ready and deal with them when we are?
    if {$subgraph_list_ == ""} {
	if {$cmd != "add_subgraph"} {
	    return;
	}
    }

    switch -exact -- $cmd {
	query_output_ids {
	    $self send_graph_comm $rid [list output_ids $output_id_list_];
	}
	query_output_info {
	    set out_id [lindex $mesg 1];
	    if {[lsearch $output_id_list_ $out_id] != -1} {
		$self send_graph_comm $rid [list output_info $out_id [list \
			[list spec $output_info_($out_id,spec)]]];
	    } else {
		$self send_graph_comm $rid [list output_info $out_id [list \
			[list error "No such output_id $out_id"]]];
	    }
	}
	set_output_spec {
	    set out_id [lindex $mesg 1];
	    set spec [lindex $mesg 2];
	    if {[lsearch $output_id_list_ $out_id] != -1} {
		$self set_output_spec $out_id $spec;
	    }
	}
	set_output_format {
	    set out_id [lindex $mesg 1];
	    set format [lindex $mesg 2];
	    if {[lsearch $output_id_list_ $out_id] != -1} {
		foreach s $subgraph_list_ {
		    $self send_to_subgraph $s -1 [list set_output_format $out_id $format] 1000 1;
		}
	    }
	}
	set_output_geometry {
	    if {[llength $mesg] == 3} {
		set out_id [lindex $mesg 1];
		set geometry [lindex $mesg 2];
		if {[lsearch $output_id_list_ $out_id] != -1} {
		    foreach s $subgraph_list_ {
			$self send_to_subgraph $s -1 [list set_output_geometry $out_id $geometry] 1000 1;
		    }
		}
	    } else {
		set geometry [lindex $mesg 2];
		foreach s $subgraph_list_ {
		    $self send_to_subgraph $s -1 [list set_output_geometry $geometry] 1000 1;
		}
	    }
	}
	add_subgraph {
	    $self instvar subgraph_info_;
	    $self instvar callback_array_;

	    set new_subgraph [lindex $mesg 1];

	    foreach sid $subgraph_list_ {
		if {$subgraph_info_($sid,spec) == $new_subgraph} {
		    return;
		}
	    }

	    incr sid_;

	    lappend subgraph_list_ $sid_;
	    set subgraph_info_($sid_,comm_obj) [new AnnounceListenManager/GraphComm [$self get_option mtu] $new_subgraph];
	    $subgraph_info_($sid_,comm_obj) register $id_ "$self recv_subgraph_comm $sid_ ";
	    set subgraph_info_($sid_,spec) $new_subgraph;

	    if {[llength $subgraph_list_] == 1} {
		set output_id_request [$self send_to_subgraph $sid_ -1 "query_output_ids" 100];
		set callback_array_($sid_,$output_id_request) "$self setup_outputs $sid_ $output_id_request ";
	    } else {
		foreach o $output_id_list_ {
		    after 1000 $self send_to_subgraph $sid_ -1 [list [list set_output_spec $o $output_info_($o,in_spec) 1000 1]];		
		}
	    }
	}
	del_subgraph {
	    set subgraph_to_del [lindex $mesg 1];
	    
	    set sid_to_del "";
	    foreach s $subgraph_list_ {
		if {$subgraph_info_($s,spec) == $subgraph_to_del} {
		    set sid_to_del $s;
		}
	    }
	    if {$sid_to_del == ""} {
		return;
	    }
	    delete $subgraph_info_($sid_to_del,comm_obj);
	    unset subgraph_info_($sid_to_del,comm_obj);
	    unset subgraph_info_($sid_to_del,spec);

	    set idx [lsearch $subgraph_list_ $sid_to_del];
	    set subgraph_list_ [lreplace $subgraph_list_ $idx $idx];

	    if {$subgraph_list_ == ""} {
		# All the subgraphs are gone. 
		# XXX Need to handle this case.
	    }
	}
    }
}

TemporalInterleaver instproc set_output_spec {out_id spec} {
    eval $self instvar [$self info vars];

    if {$output_info_($out_id,spec) == $spec} {
	return;
    }

    set inter_obj $output_info_($out_id,inter_obj);

    set spec_split [split $spec "/"];

    set addr [lindex $spec_split 0];
    set port [lindex $spec_split 1];

    set output_info_($out_id,out_vagent) [new PsvpVideoAgent $addr/$port];

    $inter_obj ssrc [$output_info_($out_id,out_vagent) get_local_srcid];
    $inter_obj target [$output_info_($out_id,out_vagent) get_transmitter];

    # Total hack XXXXX
    $output_info_($out_id,out_vagent) local_bandwidth 30000000

    set output_info_($out_id,spec) $spec;
}

Class TemporalInterleaverVideoAgent -superclass VideoAgent

TemporalInterleaverVideoAgent instproc init {spec combiner} {
    eval $self next $self $spec;

    $self instvar combiner_;

    set combiner_ $combiner;
}

TemporalInterleaverVideoAgent instproc create_decoder src {
    $self instvar combiner_;

    set d [$src handler];
    if {$d != ""} {
	delete $d;
    }

    return $combiner_;
}

TemporalInterleaverVideoAgent instproc deactivate src {
    puts "$src has been deactivated!!!!";
}


TemporalInterleaver instproc GenerateSpec {} {
    # OK, this is a REAL hack

    global spec_generator;

    if {![info exists spec_generator(init)]} {
	set spec_generator(init) 1;
	set spec_generator(b1) 224;
	set spec_generator(b2) 5;
	set spec_generator(b3) 7;
	set spec_generator(b4) 1;
	set spec_generator(port) 22334;
    }

    incr spec_generator(b4);

    return "$spec_generator(b1).$spec_generator(b2).$spec_generator(b3).$spec_generator(b4)/$spec_generator(port)/16";
}


# Some routines to help with ui in tinter.tcl

TemporalInterleaver instproc large_factor {value} {
    
    eval $self instvar [$self info vars];

    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    $inter_obj large_factor $value;
	}
    }
}

TemporalInterleaver instproc small_factor {value} {
    
    eval $self instvar [$self info vars];

    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    $inter_obj small_factor $value;
	}
    }
}

TemporalInterleaver instproc expected_tolerance_factor {value} {
    
    eval $self instvar [$self info vars];

    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    $inter_obj expected_tolerance_factor $value;
	}
    }
}

TemporalInterleaver instproc catch_up_factor {value} {
    
    eval $self instvar [$self info vars];

    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    $inter_obj catch_up_factor $value;
	}
    }
}

TemporalInterleaver instproc large_factor {value} {
    
    eval $self instvar [$self info vars];

    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    $inter_obj large_factor $value;
	}
    }
}

TemporalInterleaver instproc get_marked {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set marked_];
	}
    }
    return $res;
}

TemporalInterleaver instproc get_unmarked {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set unmarked_];
	}
    }
    return $res;
}

TemporalInterleaver instproc get_late {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set late_];
	}
    }
    return $res;
}

TemporalInterleaver instproc get_early_expected {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set early_expected_];
	}
    }
    return $res;
}

TemporalInterleaver instproc get_early_queued {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set early_queued_];
	}
    }
    return $res;
}

TemporalInterleaver instproc get_late_valid {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set late_valid_];
	}
    }
    return $res;
}

TemporalInterleaver instproc get_q_length {} {
    eval $self instvar [$self info vars];
    set res "";
    
    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    lappend res [$inter_obj set q_length_];
	}
    }
    return $res;
}


TemporalInterleaver instproc reset_stats {} {
    $self instvar output_id_list_ output_info_

    foreach o $output_id_list_ {
	set inter_obj $output_info_($o,inter_obj);
	if {$inter_obj != ""} {
	    $inter_obj reset_stats
	}
    }
}
    