
import RTPApplication
import AnnounceListenManager/GraphComm

Class BasicUIApp -superclass RTPApplication

BasicUIApp instproc init args {
    $self next one_proc_test

    $self init_resources;

    [$self options] register_option -cntrl_spec cntrl_spec
    [$self options] register_option -comm_id comm_id

    eval [$self options] parse_args $args;

    $self instvar cntrl_obj_;
    
    set cntrl_obj_ [new AnnounceListenManager/GraphComm [$self get_option mtu] [$self get_option cntrl_spec]];
    $cntrl_obj_ register [$self get_option comm_id] "$self recv_graph_comm";
    
    frame .in_frame;
    frame .out_frame;
    frame .param_frame;

    pack .in_frame -side top -fill both -expand 1
    pack .out_frame -side top  -fill both -expand 1
    pack .param_frame -side top  -fill both -expand 1

    label .in_frame.l -text "Inputs"
    label .out_frame.l -text "Outputs"
    label .param_frame.l -text "Parameters"

    pack .in_frame.l -side top -fill both -expand 1
    pack .out_frame.l -side top -fill both -expand 1
    pack .param_frame.l -side top -fill both -expand 1


    set input_id_request [$self send_graph_comm -1 "query_input_ids" 100];
    $self set callback_array_($input_id_request) "$self BuildInputFrame $input_id_request "

    set output_id_request [$self send_graph_comm -1 "query_output_ids" 100]
    $self set callback_array_($output_id_request) "$self BuildOutputFrame $output_id_request " 

    set param_id_request [$self send_graph_comm -1 "query_parameter_ids" 100]
    $self set callback_array_($param_id_request) "$self BuildParamFrame $param_id_request "
}

BasicUIApp instproc init_resources {} {
    $self add_option network ip
    $self add_option mtu 1024
    $self add_option defaultTTL 32
    $self add_option sessionType rtpv2
    $self add_option maxVideoSessionBW 30000000
}

BasicUIApp instproc send_graph_comm {rid mesg {resend_period 0} {deliver_once_flag 0}} {
    $self instvar cntrl_obj_;

    set comm_id [$self get_option comm_id];

    return [$cntrl_obj_ do_announce $comm_id $rid $mesg $resend_period $deliver_once_flag];
}

BasicUIApp instproc recv_graph_comm {rid mesg} {
    $self instvar input_id_request_;
    $self instvar callback_array_;

    if {[info exists callback_array_($rid)]} {
	eval $callback_array_($rid) [list $mesg];
	unset callback_array_($rid);
    }
}

BasicUIApp instproc BuildInputFrame {req_id reply} {
    $self instvar cntrl_obj_;

    $cntrl_obj_ stop_resend $req_id;

    set in_list [lindex $reply 1];
    
    global trig_array;
    
    foreach id $in_list {
	set f [frame .in_frame.$id];
	label $f.l -text "$id";
	entry $f.e;
	button $f.b -text "Set" -command "$self send_graph_comm -1 \[list set_input_spec $id \[$f.e get\]\] 1000 1";
	checkbutton $f.trig -text "AutoTrigger" -variable trig_array($f.trig) -command "$self send_graph_comm -1 \[list set_input_trigger $id \$trig_array($f.trig)\]"
	pack $f.l -side left;
	pack $f.e -side left -fill x -expand 1;
	pack $f.b -side left;
	pack $f.trig -side right;
	pack $f -side top -fill both -expand 1;
    }
}

BasicUIApp instproc BuildOutputFrame {req_id reply} {
    $self instvar cntrl_obj_;

    $cntrl_obj_ stop_resend $req_id;

    set out_list [lindex $reply 1];
    foreach id $out_list {
	set f [frame .out_frame.$id];
	label $f.l -text "$id";
	entry $f.e;
	button $f.b -text "Set" -command "$self send_graph_comm -1 \[list set_output_spec $id \[$f.e get\]\] 1000 1";
	pack $f.l -side left;
	pack $f.e -side left -fill x -expand 1;
	pack $f.b -side right;
	pack $f -side top -fill both -expand 1;
    }
}

BasicUIApp instproc BuildParamFrame {req_id reply} {

    $self instvar cntrl_obj_;

    $cntrl_obj_ stop_resend $req_id;

    set param_list [lindex $reply 1];
	
    foreach id $param_list {
	set param_info_request [$self send_graph_comm -1 [list query_parameter_info $id]];
	$self set callback_array_($param_info_request) "$self BuildParam $param_info_request ";
    }
}

BasicUIApp instproc BuildParam {req_id reply} {
    $self instvar cntrl_obj_;

    $cntrl_obj_ stop_resend $req_id;

    set id [lindex $reply 1];
    set pinfo [lindex $reply 2];

    set ptype "";
    set pdomain "";
    set pvalue "";

    foreach entry $pinfo {
	set e_type [lindex $entry 0];
	set e_val [lindex $entry 1];

	if {$e_type == "type"} {
	    set ptype $e_val;
	} elseif {$e_type == "domain"} {
	    set pdomain $e_val;
	} elseif {$e_type == "value"} {
	    set pvalue $e_val;
	}
    }

    set f [frame .param_frame.$id];
    label $f.l -text "$id";
    pack $f.l -side left;

    if {$ptype == "real"} {
	# Handle real value parameters here.
	set pdomain [split $pdomain "(),"];
	set from [lindex $pdomain 1];
	set to [lindex $pdomain 2];

	scale $f.s -from $from -to $to -resolution [expr ($to - $from)/100.0] -orient h;
	$f.s set $pvalue;
	pack $f.s -side right -fill x -expand 1;
	$f.s configure -command "$self send_graph_comm -1 \[list set_parameter $id \[$f.s get\]\]; #";
    } else {
	# All other types
	entry $f.e;
	button $f.b -text "Set" -command "$self send_graph_comm -1 \[list set_parameter $id \[$f.e get\]\]";
	pack $f.e -side left -fill x -expand 1;
	pack $f.b -side right;
	$f.e insert 0 $e_val;
    }

    pack $f -side top -fill both -expand 1;
}


set app [new BasicUIApp $argv];