#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/rvic-client/ui-client.tcl,v 1.11 1998/08/21 20:04:14 hodes Exp $
#


#
# client for rvic
#
Class RvicClientUI

#
# constuctor takes `w', a window to build itself in, and
# `addrSpec', the address spec of the server to talk to.
#
RvicClientUI public init {w addrspec} {
	$self instvar al_ layouts_

	set origAddr $addrspec
	# if addrspec includes a name, turn it into IP addr
	set firstchar [string index $addrspec 0]
	if [string match \[a-zA-Z\] $firstchar] {
		set n [lindex [split $addrspec "/"] 0]
		set p [lindex [split $addrspec "/"] 1]
		set s [gethostbyname $n]
		if { $s == "" } {
			puts "cannot find address for '$n'..."
			return -1
		}
		set addrspec $s/$p
	}

	set al_ [new ALM/RvicClient $addrspec $self]

	# initialize with one layout -- 'Single' 1 full-screen vw
	set layouts_(Single) "0.0+0.0+1.0x1.0"

	$self build_gui $w $origAddr
}

RvicClientUI public destroy {} {
	$self instvar al_ 
    $al_ destroy
    eval [list $self] next
}

#
# builds the client gui in windows `w'.  `subtitle' is a 
# string appended to the informational bottom label, meant for showing
# to address/port of the rvic server it is connected to.
#
RvicClientUI private build_gui {w subtitle} {
	$self instvar al_ status_ w_ numSrcs_ currSrc_ numVWs_ 

	set w_ $w

	# build bottom informational label
    set il $w.infolabel
    frame $il -relief groove
    label $il.l -text "rvic client for  $subtitle"
    pack $il -side bottom -fill x -expand 1
    pack $il.l -in $il -side bottom

	# build frame for buttons representing video-windows
	frame $w.vws -width 320 -height 240 -relief groove
	pack $w.vws -side left
	set numVWs_ 0

	# build middle src list panel
	frame $w.srcs -relief groove
	label $w.srcs.l -text "Sources:" -anchor w
	pack $w.srcs -side bottom
	pack $w.srcs.l -side top -fill x -expand 1
	set numSrcs_ 0

	# build layouts menubutton, refresh button
	set m $w.layouts.menu
	menubutton $w.layouts -text Layouts... -menu $m -relief raised
	menu $m -tearoff 0
	$m add command -label Single -command "$self set_layout Single"
	pack $w.layouts -side top -expand 1 -fill x
	button $w.refresh -text "refresh all" -command "$al_ request_full_update"
	pack $w.refresh -side top -expand 1 -fill x

	# build side options panel (voice-switch, etc...)
	frame $w.opts
	pack $w.opts -side right

	bind . <q> exit

	# setup timeout rate; request current state, 
	# then start occasional re-querying
	set t [new Timer/Periodic 30000]
	$t randomize 1
	$al_ timer $t
	$al_ request_full_update
	$al_ start
}


# set local layout and then send update to the server
#
RvicClientUI public set_layout {layout} {
	$self instvar al_ currLayout_
	$self set_local_layout $layout
	$al_ announce "Set MonitorStyle $currLayout_"
	# grab vw-->cname mappings
	$al_ announce "Set VideoWindow"
}


# set current "layout" of buttons representing vwins to name in 
# param 'layout'
#
RvicClientUI public set_local_layout {layout} {
	$self instvar layouts_ currLayout_

	set i [lsearch [array get layouts_] $layout]
	if {$i == -1} {
		puts "Bad layout name: $layout"
		return
	}
	set currLayout_ $layout
	$self update_layout $layouts_($currLayout_)
}

# move to "next" layout in the layout list and announce the change to
# the server
#
RvicClientUI public next_layout {} {
	$self instvar layouts_ currLayout_ al_

	if {![info exists currLayout_]} {
		set index 0
	} else {
		set l [array get layouts_]
		set index [lsearch $l $currLayout_]
		incr index 2
		if {$index >= [llength $l]} {
			set index 0
		}
	}
	set currLayout_ [lindex [array get layouts_] $index]
	#puts "currLayout = $currLayout_"
	$self update_layout $layouts_($currLayout_)
	$al_ announce "Set MonitorStyle $currLayout_"
}


#
# update the list of possible "layouts" of the buttons representing
# remote video windows.  `layoutsList' is expected to be a list
# of the form:<br>
# layoutName {X+Y+WxH [X+Y+WxH ...] } <br>
# layoutName2 {X+Y+WxH [X+Y+WxH ...] } <br>
# ...
#
RvicClientUI private update_layouts_list {layoutList} {
	$self instvar w_ layouts_ currLayout_

	$w_.layouts.menu delete 0 end
	foreach {name values} $layoutList {
		set layouts_($name) "$values"
		#puts "- $name -=- $values -"
		$w_.layouts.menu add command -label $name \
				-command "$self set_layout $name"
	}
	# if first update, set to first layout
	if {![info exists currLayout_]} {
		$self set_local_layout "[lindex $layoutList 0]"
		puts "setting layout to [lindex $layoutList 0]"
	}
}

#
# update radio buttons src list to the strings in list srcNamesList
# <br> updates the numSrcs_ and currSrc_ instvars to match
#
RvicClientUI private update_src_list {srcNamesList} {
	$self instvar w_ numSrcs_ currSrc_

	set s $srcNamesList
	set newLen [llength $s]
	set fr $w_.srcs

	# add/delete buttons to match number of srcs
	if {$numSrcs_ < $newLen} {
		for {set i [expr $numSrcs_+1]} {$i <= $newLen} {incr i} {
			radiobutton $fr.rb$i -variable srcs -value $i
		}
	} elseif {$numSrcs_ > $newLen} {
		for {set i [expr $newLen_+1]} {$i <= $numSrcs_} {incr i} {
			destroy $fr.rb$i
		}
	} 
	set numSrcs_ $newLen

	# update contents and pack 'em
	for {set i 1} {$i <= $numSrcs_} {incr i} {
		set l [lindex $s [expr $i-1]]
		$fr.rb$i configure -text "$l" \
				-command "$self set currSrc_ \"$l\""
		pack $fr.rb$i -in $fr -side top -anchor w -padx 4
	}

	if {[info exists currSrc_] == 0} {
		set currSrc_ [lindex $s 0]
		$fr.rb1 select
	}
}

#
# update buttons representing video windows (a "layout") to list of 
# 4-tuples passed in that each represent a button.  4-tuples include
# relative position in 0.0..1.0 (-xrel, -yrel) of frame representing
# the remote "full screen" and the height and width of the individual 
# button, again each relative in 0.0..1.0 <br>
# ... thus, tuple = `X+Y+HxW', like a geometry string
#
RvicClientUI private update_layout {tupleList} {
	$self instvar numVWs_ w_

	#puts "update to tupleList: $tupleList"

	set t $tupleList
	set newLen [llength $t]
	set fr $w_.vws
	
	# add/delete buttons to match number of srcs
	if {$numVWs_ < $newLen} {
		for {set i [expr $numVWs_+1]} {$i <= $newLen} {incr i} {
			button $fr.b$i -command "$self hit_layout_button $i" -text $i
		}
	} elseif {$numVWs_ > $newLen} {
		for {set i [expr $newLen+1]} {$i <= $numVWs_} {incr i} {
			destroy $fr.b$i
		}
	} 
	set numVWs_ $newLen

	# update size and place 'em
	for {set i 1} {$i <= $numVWs_} {incr i} {
		set geomList [split [lindex $tupleList [expr $i-1]] "+x"]
		foreach {x y h w} $geomList {
			place configure $fr.b$i \
					-relx $x -rely $y -relheight $h -relwidth $w
		}
	}
}

#
# user hits on layout button: sends message asking that the currently
# selected source (the radiobutton) be used as the new source for
# the video-window corresponding to the button.
#
RvicClientUI private hit_layout_button {num} {
	$self instvar currSrc_ al_

	$al_ announce "Set VideoWindow\n$num $currSrc_"
	$self set_button_label $num "[string range $currSrc_ 0 7]..."
}

#
# make button `num' have label `label'
#
RvicClientUI private set_button_label {num label} {
	$self instvar w_
	if {[info commands $w_.vws.b$num] != ""} {
		$w_.vws.b$num configure -text "$label"
	}
}

# ----

import AnnounceListenManager Timer/Periodic

#
# rvicClient message interface
#
Class ALM/RvicClient -superclass AnnounceListenManager


ALM/RvicClient public init {addrspec parent {mtu 1500}} {
    eval [list $self] next $addrspec $mtu

    $self instvar parent_ 
    set parent_ $parent
}

#
# receive updates from rvic server
#
ALM/RvicClient private recv_announcement {addr port data size} {
    #puts "Msg: $addr/$port\[$size\]: $data"
    $self instvar parent_

	if {[string first "MonitorStyleList" $data] != -1} {
		# XXX this blows away the "\n"s
        set data [lrange $data 1 end]
		# build a list of styles
		$parent_ update_layouts_list $data
		#puts "-- $data --"
    } elseif {[string first "SourceList" $data] != -1} {
        set data [split $data "\n"]
        set data [lrange $data 1 end]
		$parent_ update_src_list $data
    } elseif {[string first "MonitorStyle" $data] != -1} {
		set data [lrange $data 1 end]
		$parent_ set_local_layout $data
	} elseif {[string first "VideoWindow" $data] != -1} {
		set lines [split $data "\n"]
		set lines [lrange $lines 1 end]
		set counter 1
		foreach i $lines {
			$parent_ set_button_label $counter \
					"[string range [lrange $i 1 end] 0 7]..."
			incr counter
		}
	}
    #$parent_ refresh_ui
}

#
# alternates through all the state update messages
#
ALM/RvicClient private send_announcement {} {
	$self instvar whichMsg_

	if {![info exists whichMsg_]} {
		set whichMsg_ -1
	}
	incr whichMsg_ 1
	if {$whichMsg_ > 2} {set whichMsg_ 0}

	switch $whichMsg_ {
		0 {set msg "Get SourceList"}
		1 {set msg "Get MonitorStyleList"}
		2 {set msg "Set VideoWindow"}
	}
	$self announce $msg
}

#
# sends all messages necessary to update all state
# XXX should be in one pkt 
#
ALM/RvicClient private request_full_update {} {
	$self announce "Get MonitorStyleList"
	after 100
	$self announce "Get SourceList"
	after 100
	$self announce "Set MonitorStyle"
	after 100
	$self announce "Set VideoWindow"
}

